/*
 * Functions.java
 *
 * Created on February 3, 2006, 3:05 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package deb.math;

/**
 *
 * @author alver
 */
public class Functions {
    
    /**
     * Find the input value that minimizes the given scalar function without constraints. 
     * Only local convergence is guaranteed for non-convex functions.
     *
     * @param f The object function.
     * @param initValue The value to start the search at.
     * @param maxITer The maximum number of iterations before returning value.
     * @param par Optional parameters for the object function.
     * @return The optimal value found.
     */
    public static OptimizationResult nonLinearScalarMinimumSearch(ScalarFunction f, double initValue, int maxIter, double... par) {
        
        // Assume that the ScalarFunction does not provide its derivative, and use BFGS updated Hessian approx.
        double diffTol = 1e-6; // Step size used for differentiations.
        double B_k = numericalDoubleDerivative(f, initValue, diffTol, diffTol, par); // Jacobian approximation
        double x_k = initValue; // Current point.
        int k = 0; // Current step
        boolean finished = false; // Signal that we are finished
        double f_k = f.f(x_k, par);
        double lastStep = Double.MAX_VALUE;
        
        while ((k < maxIter) && (!finished)) {
            double p_k, x_next;
            
            f_k = f.f(x_k, par); // Current function value
            double df_k = numericalDerivative(f, x_k, diffTol, diffTol, par);
            
            if (B_k != 0) {
                //p_k = -df_k/B_k;
                double hessian = numericalDoubleDerivative(f, x_k, diffTol, diffTol, par);
                if (hessian < 0)
                    hessian=-hessian; // What strategy?
                else if (hessian == 0)
                    hessian = 1; // What strategy?
                p_k = -df_k/hessian;
            } else {
                p_k = 0;
                // ???
            }
            
            // Determine a value of alpha using golden section search:
            double alpha=1;
            /*
            // We want to minimize phi(x) = f(x_k + alpha*p_k).
            double limit = 0.00001;
            double lower=0, upper=1;
            int i=0;
            while (upper-lower > limit) {
                // Compute golden sections on current interval:
                double newLower = lower + 0.618*(upper-lower),
                        newUpper = upper - 0.618*(upper-lower);
                double lowEval = -f.f(x_k + newLower*p_k),
                        upEval = -f.f(x_k + newUpper*p_k);
                if (lowEval >= upEval)
                    lower = newLower;
                if (lowEval <= upEval)
                    upper = newUpper;
                i++;
            }
            alpha = 0.5*(upper+lower); // Mean value.
            */
            
            // Compute the next step
            x_next = x_k + alpha*p_k;
            double s_k = x_next - x_k;
            
            // // Some debug output:
            // System.out.println("k="+k+", x_next="+x_next+", s_k="+s_k+", alpha="+alpha);
            
            
            if (s_k == 0) {
                finished = true;
            }
            
            if (s_k != 0) {
                double y_k = numericalDerivative(f, x_next, diffTol, diffTol, par) - df_k;
                B_k = y_k/s_k; // BFGS, very simple for scalar systems
            }
            
            //if (s_k/lastStep < )
        
            lastStep = s_k;
            x_k = x_next;
            k++;
        }
        
        return new OptimizationResult(x_k, f_k, k<maxIter);
    }
    
    /**
     * Find an input value that gives a function value of 0.
     *
     * @param f The object function.
     * @param initValue The value to start the search at.
     * @param maxITer The maximum number of iterations before returning value.
     * @param par Optional parameters for the object function.
     * @return The optimal value found.
     */
    public static OptimizationResult nonLinearScalarZeroSearch(ScalarFunction f, double initValue, int maxIter, double... par) {
        
        // Assume that the ScalarFunction does not provide its derivative, and use Broyden's method:
        
        double B_k = numericalDerivative(f, initValue, 1e-6, 1e-6, par); // Jacobian approximation
        double x_k = initValue; // Current point.
        int k = 0; // Current step
        boolean finished = false; // Signal that we are finished
        double r_k = f.f(x_k, par);
        double lastStep = Double.MAX_VALUE;
        
        while ((k < maxIter) && (!finished)) {
            double p_k, x_next;
            
            r_k = f.f(x_k, par); // Current function value
            
            if (B_k != 0) {
                p_k = -r_k/B_k;
            } else {
                p_k = 0;
                // ???
            }
            
            // Determine a value of alpha using golden section search:
            double alpha=1;
            // We want to minimize phi(x) = f(x_k + alpha*p_k).
            double limit = 0.00001;
            double lower=0, upper=1;
            int i=0;
            while (upper-lower > limit) {
                // Compute golden sections on current interval:
                double newLower = lower + 0.618*(upper-lower),
                        newUpper = upper - 0.618*(upper-lower);
                double lowEval = -f.f(x_k + newLower*p_k, par),
                        upEval = -f.f(x_k + newUpper*p_k, par);
                if (lowEval >= upEval)
                    lower = newLower;
                if (lowEval <= upEval)
                    upper = newUpper;
                i++;
            }
            alpha = 0.5*(upper+lower); // Mean value.
            
            // Compute the next step
            x_next = x_k + alpha*p_k;
            double s_k = x_next - x_k;
            
            //// Some debug output:
            //System.out.println("k="+k+", x_next="+x_next+", s_k="+s_k+", alpha="+alpha+", i="+i);
            
            
            if (s_k == 0) {
                finished = true;
            }
            
            if (s_k != 0) {
                double y_k = f.f(x_next, par) - r_k;
                B_k = B_k + (y_k - B_k*s_k)/s_k;
            }
            
            //if (s_k/lastStep < )
        
            lastStep = s_k;
            x_k = x_next;
            k++;
        }
        
        return new OptimizationResult(x_k, r_k, k<maxIter);
    }
    
    /**
     * Computes numerical derivative of a scalar function.
     *
     * @param f The function.
     * @param x The point at which to differentiate.
     * @param relStep The step size to use for computing differences.
     * @param absStep The step size to use if x=0.
     */
    public static double numericalDerivative(ScalarFunction f, double x, double relStep, double absStep, double... par) {
        double step;
        if (x != 0) {
            step = relStep*x;
        } else {
            step = absStep;
        }
        double lower = f.f(x-step, par),
                upper = f.f(x+step, par);
        return (upper - lower)/(2*step);
    }
    
    /**
     * Computes numerical double derivative of a scalar function.
     *
     * @param f The function.
     * @param x The point at which to differentiate.
     * @param relStep The step size to use for computing differences.
     * @param absStep The step size to use if x=0.
     */
    public static double numericalDoubleDerivative(ScalarFunction f, double x, double relStep, double absStep, double... par) {
        double step;
        if (x != 0) {
            step = relStep*x;
        } else {
            step = absStep;
        }
        double center = f.f(x, par),
                lower = f.f(x-step, par),
                upper = f.f(x+step, par);
        return (upper - 2*center + lower)/(step*step);
    }
    
    
    
    public static void main(String[] args) {
        ScalarFunction f = new ScalarFunction() {
            public double f(double x, double... params) {
                //return Math.cos(x)+Math.sin(x);
                return (x)*(x)*x - 3*x;
            }
            public double getDerivative(double x) {
                return 0;
            }
            public boolean hasDerivative() {
                return false;
            }
        };
        
        System.out.println(nonLinearScalarMinimumSearch(f, 0.1, 50).getOptimum());
    }
}
