package deb.paramtools;

import java.io.File;
import java.io.IOException;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Created by IntelliJ IDEA.
 * User: alver
 * Date: Aug 21, 2006
 * Time: 11:10:40 PM
 * To change this template use File | Settings | File Templates.
 */
public class SetSimpleParValue implements ParamFileModifier {
    private String parName;
    private double value;
    private boolean overrideComplexParams;
    private String pattern, complPattern;
    private Pattern pat, pat2;
    private String replacementString;



    public SetSimpleParValue(String parName, double value, boolean overrideComplexParams) {

        this.parName = parName;
        this.value = value;
        this.overrideComplexParams = overrideComplexParams;

        pattern = "<parameter name=\""+parName+"\".*/>";
        complPattern = "<parameter name=\""+parName+"\".*?</parameter>";
        replacementString = "<parameter name=\""+parName+"\" value=\""+Util.format(value, 6)
                +"\"/>";
        pat = Pattern.compile(pattern);
        pat2 = Pattern.compile(complPattern, Pattern.DOTALL);

    }


    /**
     * Make the modification.
     * @param file The file to modify.
     * @return true if the modification could be made, false otherwise. The modification cannot be made
     *   if the expected pattern is not found.
     * @throws IOException
     */
    public boolean modifyFile(File file) throws IOException {
        StringBuffer contents = Util.readFile(file);
        Matcher m = pat.matcher(contents);
        boolean madeReplacement = false;
        StringBuffer newContents = new StringBuffer();
        while (m.find()) {
            m.appendReplacement(newContents, replacementString);
            madeReplacement = true;
        }
        m.appendTail(newContents);

        // The following block does the same for more complex parameters. This is optional, because
        // the operation could delete a fairly complex part of the XML file (array etc.).
        if (overrideComplexParams) {
            String middle = newContents.toString();
            newContents = new StringBuffer();
            m = pat2.matcher(middle);
            //System.out.println(middle);

            while (m.find()) {

                m.appendReplacement(newContents, replacementString);
                madeReplacement = true;
            }
            m.appendTail(newContents);
        }

        Util.writeFile(file, newContents.toString());

        return madeReplacement;
    }

    /**
     * Parse a text in the following format (example):
     *
     * /home/alver/simulation/param.xml
     * k_1  1.0 2.0 3.0
     * k_2  0.5 1.5 2.5
     *
     * These contents should result in a list of three lists of ParamFileModifiers. The first
     * list should contain (k_1 -> 1.0) and (k_2 -> 0.5). The second (k_1 -> 2.0) and (k_2 -> 1.5),
     * and the third (k_1 -> 3.0) and (k_2 -> 2.5). Iterating over the outer list, the result can
     * be three simulations in which the parameters k_1 and k_2 take on three sets of values.
     *
     * @param s The String containing the information to be parsed.
     * @param overrideComplexParams Indicate whether we should allow overwriting complex parameters (arrays etc.)
     * @return the parsed lists of changes, or null if parsing failed.
     */
    public static List<List<ParamFileModifier>> parseChangeList(String s, boolean overrideComplexParams) {
        List<List<ParamFileModifier>> list = new ArrayList<List<ParamFileModifier>>();
        String[] lines = s.split("\n");
        if (lines.length < 2) {
            System.err.println("Two or more lines of information are needed to parse change list.");
            return null;
        }
        File file = new File(lines[0].trim());
        ArrayList<String[]> lineParts = new ArrayList<String[]>();
        int subparts = 99999;
        for (String line1 : lines) {
            String line = line1.trim();
            if (line.length() > 0) {
                String[] split = line.split("\t");
                lineParts.add(split);
                if (split.length - 1 < subparts)
                    subparts = split.length - 1;
            }
        }
        // subparts now indicates what is the smallest number of items in one of the lines.
        for (int i=0; i<subparts; i++) {
            ArrayList<ParamFileModifier> thisList = new ArrayList<ParamFileModifier>();
            list.add(thisList);
            for (String[] elems : lineParts) {
                try {
                    double value = Double.parseDouble(elems[i + 1]);
                    thisList.add(new SetSimpleParValue(elems[0], value, overrideComplexParams));
                } catch (NumberFormatException ex) {
                    System.err.println("Could not parse number: " + elems[i + 1]);
                    return null;
                }
            }
        }

        return list;
    }


    public static void main(String[] args) {

        String parseText = "/home/alver/hjemmek/div/test.xml\n"
                +"";
        File source = new File("/home/alver/hjemmek/div/test.xml");
        File backup = new File("/home/alver/hjemmek/div/test_bak.xml");
        try {
            Util.copyFile(source, backup, false);

            SetSimpleParValue mod = new SetSimpleParValue("baseFeedReference", 45.4, true);
            System.out.println(mod.modifyFile(source));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
