package deb.paramtools;

import java.io.*;
import java.text.NumberFormat;
import java.util.List;

/**
 *
 */
public class Util {

    private static final String LINE_SEPARATOR = (System.getProperty("os.name").toLowerCase().startsWith("windows") ? "\r\n" : "\n");

    private static NumberFormat format = NumberFormat.getNumberInstance();

    /**
     * Perform a list of modifications to the given file.
     * @param file The file to modify.
     * @param changes The List of changes.
     * @param stopAfterFailure Indicates whether the method should return immediately if one change fails, or
     *          if it should proceed to run the remaining ones.
     * @return true if all changes succeeded, false otherwise.
     */
    public static boolean doParameterChanges(File file, List<ParamFileModifier> changes, boolean stopAfterFailure) throws IOException {
        boolean success = true;
        for (ParamFileModifier modifier : changes) {
            boolean status = modifier.modifyFile(file);
            if (!status) {
                if (stopAfterFailure)
                    return false;
                else
                    success =false;
            }
        }
        return success;
    }

    
    /**
     * Returns a String representation of the given number, formatted with the given maximum
     * number of digits.
     * @param value The number to format.
     * @param maxFractionDigits The maximum decimals in the formatted number.
     * @return The formatted number.
     */
    public static String format(double value, int maxFractionDigits) {
        format.setMaximumFractionDigits(maxFractionDigits);
        return format.format(value);
    }

    /**
     * Create a file copy in a temporary file, and return its handle.
     * @param source The file to copy.
     * @return The reference to the temporary file.
     */
    public static File prepareTempFile(File source) throws IOException {
        File temp = File.createTempFile("deblife", "");
        temp.deleteOnExit();
        boolean success = copyFile(source, temp, true);
        if (success)
            return temp;
        else
            return null;
    }

    public static Streams openStreams(File from, File to) throws IOException {
        Reader in = new FileReader(from);
        Writer out = new FileWriter(to);
        return new Streams(in, out);
    }

    /**
     * Read the contents of the file.
     * @param file The file to read.
     * @return A StringBuffer containing the entire file contents. Line breaks are represented as '\n' only.
     */
    public static StringBuffer readFile(File file) throws IOException {
        BufferedReader in = null;
        StringBuffer sb = new StringBuffer();
        try {
            in = new BufferedReader(new FileReader(file));
            String line;
            while ((line = in.readLine()) != null) {
                sb.append(line);
                sb.append('\n');
            }
            return sb;
        } finally {
            if (in != null)
                in.close();
        }
    }

    /**
     * Write a string to file.
     * @param file The file to write to.
     * @param s The string to write.
     * @throws IOException
     */
    public static void writeFile(File file, String s) throws IOException {
        BufferedWriter out = null;
        try {
            out = new BufferedWriter(new FileWriter(file));
            for (int i=0; i<s.length(); i++) {
                if (s.charAt(i) == '\n')
                    out.write(LINE_SEPARATOR);
                else
                    out.write(s.charAt(i));
            }
        } finally {
            if (out != null)
                out.close();
        }
    }

    /**
     * Copies a file.
     *
     * @param source         File Source file
     * @param dest           File Destination file
     * @param deleteIfExists boolean Determines whether the copy goes on even if the file
     *                       exists.
     * @return boolean Whether the copy succeeded, or was stopped due to the
     *         file already existing.
     * @throws java.io.IOException
     */
    public static boolean copyFile(File source, File dest, boolean deleteIfExists)
            throws IOException {

        BufferedInputStream in = null;
        BufferedOutputStream out = null;
        try {
            // Check if the file already exists.
            if (dest.exists()) {
                if (!deleteIfExists)
                    return false;
                // else dest.delete();
            }

            in = new BufferedInputStream(new FileInputStream(source));
            out = new BufferedOutputStream(new FileOutputStream(dest));
            int el;
            // int tell = 0;
            while ((el = in.read()) >= 0) {
                out.write(el);
            }
        } catch (IOException ex) {
            throw ex;
        } finally {
            if (out != null) {
                out.flush();
                out.close();
            }
            if (in != null)
                in.close();
        }
        return true;
    }


    public static class Streams {
        private Reader in;
        private Writer out;

        public Streams(Reader in, Writer out) {

            this.in = in;
            this.out = out;
        }

        public Reader getIn() {
            return in;
        }

        public void setIn(Reader in) {
            this.in = in;
        }

        public Writer getOut() {
            return out;
        }

        public void setOut(Writer out) {
            this.out = out;
        }
    }
}
